#include <avr/io.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include "adc.h"


//Przeliczone wartoi ADC dla temperatur od 10'C...34'C (ostatnia warto dodana z uwagi na mechanizm wyszukiwania indexu w funkcji ADC)
const uint16_t tc1047TransferFunction[26] PROGMEM = {563, 572, 581, 591, 600, 609, 619, 628, 637, 646, 656, 665, 674, 684, 693, 702, 712, 721, 730, 740, 749, 758, 768, 777, 786, 1023};


//#############################################################################################

uint8_t readADC(uint8_t Channel)  //6.8 ms
{
		uint8_t Index;
		uint16_t adcVal = 0, currVal, nextVal;


		//Konfiguracja przetwornika ADC: Referencja = 1.1V, wejcie specyfikowane wartoci zmiennej Channel
		ADMUX = (1<<REFS1)|Channel;

		//Wykonanie 32 pomiarw ADC w celu pniejszego urednienia wartoci wynikowej
		for(uint8_t i=0; i<32; ++i)
		{
				ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADPS2);  //Start konwersji, Prescaler= 16 (62.5 kHz)
				while(ADCSRA & (1<<ADSC));  //Czekamy na jej zakoczenie - 210us
				adcVal += ADC;
		}

		//Urednienie wartoci wynikowej
		adcVal /= 32;

		//W zalenoci od mierzonego kanau warto funkcji liczona jest wedug innego algorytmu
		if(Channel == BRIGHTNESS)
		{
				return adcVal>>5;  //Zakres 0...31
		}
		else  //TEMPERATURE
		{
				//Ograniczamy warto od dou
				if(adcVal < 558) adcVal = 558;

				//Wyszukujemy warto napicia (jego index) w tablicy TC1047
				for(Index = 0; Index < 25; ++Index)
				{
						currVal = pgm_read_word(&tc1047TransferFunction[Index]);
						nextVal = pgm_read_word(&tc1047TransferFunction[Index+1]);

						if(adcVal >= currVal && adcVal < nextVal) break;
				}
				return Index;
		}
}

//#############################################################################################

