//termometrRGB. Copyright Robert Wogajew, 2023. ATtiny25 @ 1MHz
//03.01.2023: Gotowy bez sprawdzania na ukadzie docelowym
//04.01.2023: zmieniono spos pomiaru temperatury, odwieanie w ptli gwnej i MODE przeczono z pinu reset na PB0
//Dodano demo
#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <inttypes.h>
#include "ADC/adc.h"
#include "APA102/apa102.h"
#include "defines.h"


uint8_t Refresh;  //Zmienna dania odwierzenia ekranu
uint8_t Brightness;  //Globalna jasno wywietlacza LED
uint8_t Mode EEMEM;  //Tryb wywietlania temperatury w EEPROM

//Stae skadowych RGB dla kolejnych wartoci mierzonej temperatury (10...34 'C)
uint8_t B[25] = {255, 234, 213, 192, 171, 150, 129, 108, 87, 66, 45, 24, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 	0, 	0, 	0};
uint8_t G[25] = {0, 21, 42, 63, 84, 105, 126, 147, 168, 189, 210, 231, 255, 231, 210, 189, 168, 147, 126, 105, 84, 63, 42, 21, 0};
uint8_t R[25] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 24, 45, 66, 87, 108, 129, 150, 171, 192, 213, 234, 255};

void handleKeyboard(void);
void refreshDisplay(void);
void readBrightness(void);
void Demo(void);

//#############################################################################################
//#############################################################################################
//#############################################################################################

int main(void)
{
		uint8_t Timer = 0;

		//Podcignicie przycisku MODE pod VCC
		MODE_PULLED_UP;
		//Inicjalizacja portw sterujcych diodami LED
		APA102Init();
		//Proste demo na wstpie
		Demo();


		while(1)
		{
				//Co okoo sekund mierzymy i wywietlamy temperatur
				if((++Timer%128) == 0) Refresh = 1;
				//Obsuga klawiatury zmieniajcej sposb prezentacji temperatury
				handleKeyboard();
				//Obsuga potencjometru odpowiedzialnego za regulacj jasnoci linijki LED
				readBrightness();  //Czas wykonania ok. 7 ms
				//Odwierzenie ekranu, czyli wywietlenie biecej temperatury wedug biecych kryteriw wywietlania
				refreshDisplay();
		}
}

//#############################################################################################
//#############################################################################################
//#############################################################################################

void Demo(void)
{
		//Proste demo
		for(uint8_t j=0; j<25; ++j)
		{
				APA102StartTrans();
				for(uint8_t i=0; i<25; ++i) if(i <= j) APA102SendRGBdata(31, R[i], G[i], B[i]); else APA102SendRGBdata(0, 0, 0, 0);
				APA102StopTrans(25);
				_delay_ms(25);
		}
		_delay_ms(500);
		for(uint8_t j=0; j<25; ++j)
		{
				APA102StartTrans();
				for(uint8_t i=0; i<25; ++i) if(i <= j) APA102SendRGBdata(0, 0, 0, 0); else APA102SendRGBdata(31, R[i], G[i], B[i]);
				APA102StopTrans(25);
				_delay_ms(25);
		}
		_delay_ms(500);
}

//#############################################################################################

void handleKeyboard(void)
{
		if(MODE_IS_PRESSED)
		{
				_delay_ms(35);
				if(MODE_IS_PRESSED)
				{
						if(eeprom_read_byte(&Mode) != MODE_COLOR) eeprom_write_byte(&Mode, MODE_COLOR); else eeprom_write_byte(&Mode, MODE_RGB);
						Refresh = 1;  //Odwierzenie ekranu
				}
		}
}

//#############################################################################################

void readBrightness(void)
{
		static uint8_t lastBrightness;

		Brightness = readADC(BRIGHTNESS);
		if(lastBrightness != Brightness)
		{
				lastBrightness = Brightness;
				Refresh = 1;
		}
}

//#############################################################################################

void refreshDisplay(void)
{
		uint8_t Temperature;  //0...24

		if(Refresh)
		{
				Refresh = 0;

				Temperature = readADC(TEMPERATURE);  //0...24

				//W zalenoci od sposobu prezentacji temperatury wysyamy odpowiednie wartoci do diod APA102
				APA102StartTrans();
				if(eeprom_read_byte(&Mode) != MODE_COLOR)  //RGB
				{
						for(uint8_t i=0; i<25; ++i) if(i <= Temperature) APA102SendRGBdata(Brightness, R[i], G[i], B[i]); else APA102SendRGBdata(0, 0, 0, 0);
				}
				else  //COLOR
				{
						for(uint8_t i=0; i<25; ++i) if(i <= Temperature) APA102SendRGBdata(Brightness, R[Temperature], G[Temperature], B[Temperature]); else APA102SendRGBdata(0, 0, 0, 0);
				}
				APA102StopTrans(25);

				//Dodane wycznie z uwagi na prosty sposb eliminacji drgania stykw w funkcji handleKeyboard()
				_delay_ms(35);
		}
}

//#############################################################################################
